@@ -1,22 +1,7 @@
+// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 // -*- Mode: C++ -*-
 //
 // Copyright (C) 2013-2020 Red Hat, Inc.
-//
-// This file is part of the GNU Application Binary Interface Generic
-// Analysis and Instrumentation Library (libabigail).  This library is
-// free software; you can redistribute it and/or modify it under the
-// terms of the GNU Lesser General Public License as published by the
-// Free Software Foundation; either version 3, or (at your option) any
-// later version.
-
-// This library is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// General Lesser Public License for more details.
-
-// You should have received a copy of the GNU Lesser General Public
-// License along with this program; see the file COPYING-LGPLV3.  If
-// not, see <http://www.gnu.org/licenses/>.
 
 /// @file
 ///
@@ -36,14 +21,14 @@
 #ifndef __ABG_DIFF_UTILS_H__
 #define __ABG_DIFF_UTILS_H__
 
-#include <stdexcept>
 #include <cassert>
 #include <cstdlib>
+#include <memory>
 #include <ostream>
+#include <sstream>
+#include <stdexcept>
 #include <string>
 #include <vector>
-#include <sstream>
-#include "abg-cxx-compat.h"
 #include "abg-fwd.h"
 
 namespace abigail
@@ -57,7 +42,7 @@ namespace abigail
 namespace diff_utils
 {
 
-using abg_compat::shared_ptr;
+using std::shared_ptr;
 
 // Inject the names from std:: below into this namespace
 using std::string;
