@@ -1,23 +1,8 @@
+// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 // -*- Mode: C++ -*-
 //
 // Copyright (C) 2016-2020 Red Hat, Inc.
 //
-// This file is part of the GNU Application Binary Interface Generic
-// Analysis and Instrumentation Library (libabigail).  This library is
-// free software; you can redistribute it and/or modify it under the
-// terms of the GNU Lesser General Public License as published by the
-// Free Software Foundation; either version 3, or (at your option) any
-// later version.
-
-// This library is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// General Lesser Public License for more details.
-
-// You should have received a copy of the GNU Lesser General Public
-// License along with this program; see the file COPYING-LGPLV3.  If
-// not, see <http://www.gnu.org/licenses/>.
-//
 // Author: Dodji Seketeli
 
 /// @file
@@ -31,16 +16,17 @@
 #ifndef __ABG_INTERNED_STR_H__
 #define __ABG_INTERNED_STR_H__
 
+#include <functional>
+#include <memory>
 #include <ostream>
 #include <string>
-#include "abg-cxx-compat.h"
+#include <unordered_set>
 
 
 namespace abigail
 {
 // Inject some std types into this namespace.
-using abg_compat::unordered_set;
-using abg_compat::shared_ptr;
+using std::unordered_set;
 using std::string;
 using std::ostream;
 
@@ -226,7 +212,7 @@ struct hash_interned_string
   size_t
   operator()(const interned_string& s) const
   {
-    abg_compat::hash<size_t> hash_size_t;
+    std::hash<size_t> hash_size_t;
     return hash_size_t(reinterpret_cast<size_t>(s.raw()));
   }
 }; // end struct hash_interned_string
@@ -240,9 +226,7 @@ struct hash_interned_string
 class interned_string_pool
 {
   struct priv;
-  typedef shared_ptr<priv> priv_sptr;
-
-  priv_sptr priv_;
+  std::unique_ptr<priv> priv_;
 
 public:
 
