@@ -1,22 +1,7 @@
+// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2020 Red Hat, Inc.
-//
-// This file is part of the GNU Application Binary Interface Generic
-// Analysis and Instrumentation Library (libabigail).  This library is
-// free software; you can redistribute it and/or modify it under the
-// terms of the GNU Lesser General Public License as published by the
-// Free Software Foundation; either version 3, or (at your option) any
-// later version.
-
-// This library is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// General Lesser Public License for more details.
-
-// You should have received a copy of the GNU Lesser General Public
-// License along with this program; see the file COPYING-LGPLV3.  If
-// not, see <http://www.gnu.org/licenses/>.
+// Copyright (C) 2013-2021 Red Hat, Inc.
 //
 // Author: Dodji Seketeli
 
@@ -67,16 +52,6 @@ create_native_xml_read_context(std::istream* in, environment* env);
 const string&
 read_context_get_path(const read_context&);
 
-abigail::corpus_sptr
-read_corpus_from_file(const string& path);
-
-int
-read_corpus_from_file(corpus_sptr& corp,
-		      const string& path);
-
-int
-read_corpus_from_file(corpus_sptr& corp);
-
 corpus_sptr
 read_corpus_from_native_xml(std::istream* in,
 			    environment*  env);
@@ -107,6 +82,12 @@ void
 consider_types_not_reachable_from_public_interfaces(read_context& ctxt,
 						    bool flag);
 }//end xml_reader
+
+#ifdef WITH_DEBUG_SELF_COMPARISON
+bool
+load_canonical_type_ids(xml_reader::read_context& ctxt,
+			const string& file_path);
+#endif
 }//end namespace abigail
 
 #endif // __ABG_READER_H__
