@@ -1,34 +1,19 @@
+// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2020 Red Hat, Inc.
-//
-// This file is part of the GNU Application Binary Interface Generic
-// Analysis and Instrumentation Library (libabigail).  This library is
-// free software; you can redistribute it and/or modify it under the
-// terms of the GNU Lesser General Public License as published by the
-// Free Software Foundation; either version 3, or (at your option) any
-// later version.
-
-// This library is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// General Lesser Public License for more details.
-
-// You should have received a copy of the GNU Lesser General Public
-// License along with this program; see the file COPYING-LGPLV3.  If
-// not, see <http://www.gnu.org/licenses/>.
+// Copyright (C) 2013-2021 Red Hat, Inc.
 
 ///@file
 
 #ifndef __ABG_TOOLS_UTILS_H
 #define __ABG_TOOLS_UTILS_H
 
-#include <string>
-#include <set>
-#include <ostream>
-#include <istream>
 #include <iostream>
-#include "abg-cxx-compat.h"
+#include <istream>
+#include <memory>
+#include <ostream>
+#include <set>
+#include <string>
 #include "abg-suppression.h"
 
 namespace abigail
@@ -42,7 +27,7 @@ using std::istream;
 using std::ifstream;
 using std::string;
 using std::set;
-using abg_compat::shared_ptr;
+using std::shared_ptr;
 
 const char* get_system_libdir();
 const char* get_anonymous_struct_internal_name_prefix();
@@ -150,9 +135,7 @@ typedef shared_ptr<temp_file> temp_file_sptr;
 class temp_file
 {
   struct priv;
-  typedef shared_ptr<priv> priv_sptr;
-
-  priv_sptr priv_;
+  std::unique_ptr<priv> priv_;
 
   temp_file();
 
@@ -195,9 +178,6 @@ enum file_type
   // A native abixml file format representing a corpus group of one or
   // several corpora.
   FILE_TYPE_XML_CORPUS_GROUP,
-  // A zip file, possibly containing a corpus of one of several
-  // translation units.
-  FILE_TYPE_ZIP_CORPUS,
   /// An RPM (.rpm) binary file
   FILE_TYPE_RPM,
   /// An SRPM (.src.rpm) file
@@ -267,9 +247,7 @@ abidiff_status_has_incompatible_abi_change(abidiff_status s);
 class timer
 {
   struct priv;
-  typedef shared_ptr<priv> priv_sptr;
-
-  priv_sptr priv_;
+  std::unique_ptr<priv> priv_;
 
 public:
   enum kind
@@ -318,7 +296,7 @@ bool
 file_is_kernel_debuginfo_package(const string& file_path,
 				 file_type file_type);
 
-abg_compat::shared_ptr<char>
+std::shared_ptr<char>
 make_path_absolute(const char*p);
 
 char*
