@@ -1,22 +1,7 @@
+// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2019 Red Hat, Inc.
-//
-// This file is part of the GNU Application Binary Interface Generic
-// Analysis and Instrumentation Library (libabigail).  This library is
-// free software; you can redistribute it and/or modify it under the
-// terms of the GNU Lesser General Public License as published by the
-// Free Software Foundation; either version 3, or (at your option) any
-// later version.
-
-// This library is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// General Lesser Public License for more details.
-
-// You should have received a copy of the GNU Lesser General Public
-// License along with this program; see the files COPYING-LGPLV3.  If
-// not, see <http://www.gnu.org/licenses/>.
+// Copyright (C) 2013-2020 Red Hat, Inc.
 
 /// @file
 
@@ -34,7 +19,7 @@ namespace abigail
 {
 config::config()
   : m_format_minor("0"),
-    m_format_major("1"),
+    m_format_major("2"),
     m_xml_element_indent(2),
     m_tu_instr_suffix(".bi"),
     m_tu_instr_archive_suffix(".abi")
@@ -86,11 +71,13 @@ extern "C"
 void
 abigail_get_library_version(std::string& major,
 			    std::string& minor,
-			    std::string& revision)
+			    std::string& revision,
+			    std::string& suffix)
 {
   major = ABIGAIL_VERSION_MAJOR;
   minor = ABIGAIL_VERSION_MINOR;
   revision = ABIGAIL_VERSION_REVISION;
+  suffix = ABIGAIL_VERSION_SUFFIX;
 }
 
 }
