@@ -1,26 +1,11 @@
+// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 // -*- mode: C++ -*-
 //
 // Copyright (C) 2013-2020 Red Hat, Inc.
-//
-// This file is part of the GNU Application Binary Interface Generic
-// Analysis and Instrumentation Library (libabigail).  This library is
-// free software; you can redistribute it and/or modify it under the
-// terms of the GNU Lesser General Public License as published by the
-// Free Software Foundation; either version 3, or (at your option) any
-// later version.
-
-// This library is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// General Lesser Public License for more details.
-
-// You should have received a copy of the GNU Lesser General Public
-// License along with this program; see the file COPYING-LGPLV3.  If
-// not, see <http://www.gnu.org/licenses/>.
 
 /// @file
 
-#include "abg-cxx-compat.h"
+#include <functional>
 #include "abg-internal.h"
 // <headers defining libabigail's API go under here>
 ABG_BEGIN_EXPORT_DECLARATIONS
@@ -109,8 +94,8 @@ using namespace abigail::ir;
 size_t
 type_base::hash::operator()(const type_base& t) const
 {
-  abg_compat::hash<size_t> size_t_hash;
-  abg_compat::hash<string> str_hash;
+  std::hash<size_t> size_t_hash;
+  std::hash<string> str_hash;
 
   size_t v = str_hash(typeid(t).name());
   v = hashing::combine_hashes(v, size_t_hash(t.get_size_in_bits()));
@@ -142,7 +127,7 @@ struct decl_base::hash
   size_t
   operator()(const decl_base& d) const
   {
-    abg_compat::hash<string> str_hash;
+    std::hash<string> str_hash;
 
     size_t v = str_hash(typeid(d).name());
     if (!d.get_linkage_name().empty())
@@ -165,7 +150,7 @@ struct type_decl::hash
   {
     decl_base::hash decl_hash;
     type_base::hash type_hash;
-    abg_compat::hash<string> str_hash;
+    std::hash<string> str_hash;
 
     size_t v = str_hash(typeid(t).name());
     v = hashing::combine_hashes(v, decl_hash(t));
@@ -183,7 +168,7 @@ struct type_decl::hash
 size_t
 scope_decl::hash::operator()(const scope_decl& d) const
 {
-  abg_compat::hash<string> hash_string;
+  std::hash<string> hash_string;
   size_t v = hash_string(typeid(d).name());
   for (scope_decl::declarations::const_iterator i =
 	 d.get_member_decls().begin();
@@ -210,7 +195,7 @@ struct scope_type_decl::hash
   {
     decl_base::hash decl_hash;
     type_base::hash type_hash;
-    abg_compat::hash<string> str_hash;
+    std::hash<string> str_hash;
 
     size_t v = str_hash(typeid(t).name());
     v = hashing::combine_hashes(v, decl_hash(t));
@@ -227,7 +212,7 @@ struct qualified_type_def::hash
   {
     type_base::hash type_hash;
     decl_base::hash decl_hash;
-    abg_compat::hash<string> str_hash;
+    std::hash<string> str_hash;
 
     size_t v = str_hash(typeid(t).name());
     v = hashing::combine_hashes(v, type_hash(t));
@@ -242,7 +227,7 @@ struct pointer_type_def::hash
   size_t
   operator()(const pointer_type_def& t) const
   {
-    abg_compat::hash<string> str_hash;
+    std::hash<string> str_hash;
     type_base::hash type_base_hash;
     decl_base::hash decl_hash;
     type_base::shared_ptr_hash hash_type_ptr;
@@ -260,7 +245,7 @@ struct reference_type_def::hash
   size_t
   operator()(const reference_type_def& t)
   {
-    abg_compat::hash<string> hash_str;
+    std::hash<string> hash_str;
     type_base::hash hash_type_base;
     decl_base::hash hash_decl;
     type_base::shared_ptr_hash hash_type_ptr;
@@ -281,7 +266,7 @@ struct array_type_def::subrange_type::hash
   size_t
   operator()(const array_type_def::subrange_type& s) const
   {
-    abg_compat::hash<int> hash_size_t;
+    std::hash<int> hash_size_t;
     size_t v = hash_size_t(hash_size_t(s.get_lower_bound()));
     v = hashing::combine_hashes(v, hash_size_t(s.get_upper_bound()));
     return v;
@@ -293,7 +278,7 @@ struct array_type_def::hash
   size_t
   operator()(const array_type_def& t)
   {
-    abg_compat::hash<string> hash_str;
+    std::hash<string> hash_str;
     type_base::hash hash_type_base;
     decl_base::hash hash_decl;
     type_base::shared_ptr_hash hash_type_ptr;
@@ -320,10 +305,10 @@ struct enum_type_decl::hash
   size_t
   operator()(const enum_type_decl& t) const
   {
-    abg_compat::hash<string> str_hash;
+    std::hash<string> str_hash;
     decl_base::hash decl_hash;
     type_base::shared_ptr_hash type_ptr_hash;
-    abg_compat::hash<size_t> size_t_hash;
+    std::hash<size_t> size_t_hash;
 
     size_t v = str_hash(typeid(t).name());
     v = hashing::combine_hashes(v, decl_hash(t));
@@ -345,7 +330,7 @@ struct typedef_decl::hash
   size_t
   operator()(const typedef_decl& t) const
   {
-    abg_compat::hash<string> str_hash;
+    std::hash<string> str_hash;
     type_base::hash hash_type;
     decl_base::hash decl_hash;
     type_base::shared_ptr_hash type_ptr_hash;
@@ -371,10 +356,10 @@ struct typedef_decl::hash
 size_t
 var_decl::hash::operator()(const var_decl& t) const
 {
-  abg_compat::hash<string> hash_string;
+  std::hash<string> hash_string;
   decl_base::hash hash_decl;
   type_base::shared_ptr_hash hash_type_ptr;
-  abg_compat::hash<size_t> hash_size_t;
+  std::hash<size_t> hash_size_t;
 
   size_t v = hash_string(typeid(t).name());
   v = hashing::combine_hashes(v, hash_decl(t));
@@ -410,10 +395,10 @@ var_decl::hash::operator()(const var_decl* t) const
 size_t
 function_decl::hash::operator()(const function_decl& t) const
 {
-  abg_compat::hash<int> hash_int;
-  abg_compat::hash<size_t> hash_size_t;
-  abg_compat::hash<bool> hash_bool;
-  abg_compat::hash<string> hash_string;
+  std::hash<int> hash_int;
+  std::hash<size_t> hash_size_t;
+  std::hash<bool> hash_bool;
+  std::hash<string> hash_string;
   decl_base::hash hash_decl_base;
   type_base::shared_ptr_hash hash_type_ptr;
 
@@ -455,8 +440,8 @@ function_decl::parameter::hash::operator()
   (const function_decl::parameter& p) const
 {
   type_base::shared_ptr_hash hash_type_ptr;
-  abg_compat::hash<bool> hash_bool;
-  abg_compat::hash<unsigned> hash_unsigned;
+  std::hash<bool> hash_bool;
+  std::hash<unsigned> hash_unsigned;
   size_t v = hash_type_ptr(p.get_type());
   v = hashing::combine_hashes(v, hash_unsigned(p.get_index()));
   v = hashing::combine_hashes(v, hash_bool(p.get_variadic_marker()));
@@ -479,7 +464,7 @@ struct method_type::hash
   size_t
   operator()(const method_type& t) const
   {
-    abg_compat::hash<string> hash_string;
+    std::hash<string> hash_string;
     type_base::shared_ptr_hash hash_type_ptr;
     function_decl::parameter::hash hash_parameter;
 
@@ -515,7 +500,7 @@ struct method_type::hash
 size_t
 function_type::hash::operator()(const function_type& t) const
 {
-  abg_compat::hash<string> hash_string;
+  std::hash<string> hash_string;
   type_base::shared_ptr_hash hash_type_ptr;
   function_decl::parameter::hash hash_parameter;
 
@@ -559,7 +544,7 @@ function_type::hash::operator()(const function_type_sptr t) const
 size_t
 member_base::hash::operator()(const member_base& m) const
 {
-  abg_compat::hash<int> hash_int;
+  std::hash<int> hash_int;
   return hash_int(m.get_access_specifier());
 }
 
@@ -568,9 +553,9 @@ class_decl::base_spec::hash::operator()(const base_spec& t) const
 {
   member_base::hash hash_member;
   type_base::shared_ptr_hash hash_type_ptr;
-  abg_compat::hash<size_t> hash_size;
-  abg_compat::hash<bool> hash_bool;
-  abg_compat::hash<string> hash_string;
+  std::hash<size_t> hash_size;
+  std::hash<bool> hash_bool;
+  std::hash<string> hash_string;
 
   size_t v = hash_string(typeid(t).name());
   v = hashing::combine_hashes(v, hash_member(t));
@@ -584,10 +569,10 @@ size_t
 member_function_template::hash::operator()
   (const member_function_template& t) const
 {
-  abg_compat::hash<bool> hash_bool;
+  std::hash<bool> hash_bool;
   function_tdecl::hash hash_function_tdecl;
   member_base::hash hash_member;
-  abg_compat::hash<string> hash_string;
+  std::hash<string> hash_string;
 
   size_t v = hash_member(t);
   string n = t.get_qualified_name();
@@ -604,7 +589,7 @@ member_class_template::hash::operator()
 {
   member_base::hash hash_member;
   class_tdecl::hash hash_class_tdecl;
-  abg_compat::hash<string> hash_string;
+  std::hash<string> hash_string;
 
   size_t v = hash_member(t);
   string n = t.get_qualified_name();
@@ -642,7 +627,7 @@ class_or_union::hash::operator()(const class_or_union& t) const
 
   ABG_ASSERT(!t.get_is_declaration_only());
 
-  abg_compat::hash<string> hash_string;
+  std::hash<string> hash_string;
   scope_type_decl::hash hash_scope_type;
   var_decl::hash hash_data_member;
   member_function_template::hash hash_member_fn_tmpl;
@@ -721,7 +706,7 @@ class_decl::hash::operator()(const class_decl& t) const
 
   ABG_ASSERT(!t.get_is_declaration_only());
 
-  abg_compat::hash<string> hash_string;
+  std::hash<string> hash_string;
   class_decl::base_spec::hash hash_base;
   class_or_union::hash hash_class_or_union;
 
@@ -770,8 +755,8 @@ struct template_parameter::hash
 
     t.set_hashing_has_started(true);
 
-    abg_compat::hash<unsigned> hash_unsigned;
-    abg_compat::hash<std::string> hash_string;
+    std::hash<unsigned> hash_unsigned;
+    std::hash<std::string> hash_string;
     template_decl::hash hash_template_decl;
 
     size_t v = hash_string(typeid(t).name());
@@ -801,7 +786,7 @@ struct template_parameter::shared_ptr_hash
 size_t
 template_decl::hash::operator()(const template_decl& t) const
 {
-  abg_compat::hash<string> hash_string;
+  std::hash<string> hash_string;
   template_parameter::shared_ptr_hash hash_template_parameter;
 
   size_t v = hash_string(typeid(t).name());
@@ -822,7 +807,7 @@ struct type_tparameter::hash
   size_t
   operator()(const type_tparameter& t) const
   {
-    abg_compat::hash<string> hash_string;
+    std::hash<string> hash_string;
     template_parameter::hash hash_template_parameter;
     type_decl::hash hash_type;
 
@@ -843,7 +828,7 @@ size_t
 non_type_tparameter::hash::operator()(const non_type_tparameter& t) const
 {
   template_parameter::hash hash_template_parameter;
-  abg_compat::hash<string> hash_string;
+  std::hash<string> hash_string;
   type_base::shared_ptr_hash hash_type;
 
   size_t v = hash_string(typeid(t).name());
@@ -868,7 +853,7 @@ struct template_tparameter::hash
   size_t
   operator()(const template_tparameter& t) const
   {
-    abg_compat::hash<string> hash_string;
+    std::hash<string> hash_string;
     type_tparameter::hash hash_template_type_parm;
     template_decl::hash hash_template_decl;
 
@@ -906,7 +891,7 @@ operator()(const template_parameter* t) const
 size_t
 type_composition::hash::operator()(const type_composition& t) const
 {
-  abg_compat::hash<string> hash_string;
+  std::hash<string> hash_string;
   type_base::dynamic_hash hash_type;
 
   size_t v = hash_string(typeid(t).name());
@@ -927,7 +912,7 @@ size_t
 function_tdecl::hash::
 operator()(const function_tdecl& t) const
 {
-  abg_compat::hash<string> hash_string;
+  std::hash<string> hash_string;
   decl_base::hash hash_decl_base;
   template_decl::hash hash_template_decl;
   function_decl::hash hash_function_decl;
@@ -956,7 +941,7 @@ size_t
 class_tdecl::hash::
 operator()(const class_tdecl& t) const
 {
-  abg_compat::hash<string> hash_string;
+  std::hash<string> hash_string;
   decl_base::hash hash_decl_base;
   template_decl::hash hash_template_decl;
   class_decl::hash hash_class_decl;
