@@ -1,23 +1,8 @@
+// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 // -*- Mode: C++ -*-
 //
 // Copyright (C) 2017-2020 Red Hat, Inc.
 //
-// This file is part of the GNU Application Binary Interface Generic
-// Analysis and Instrumentation Library (libabigail).  This library is
-// free software; you can redistribute it and/or modify it under the
-// terms of the GNU Lesser General Public License as published by the
-// Free Software Foundation; either version 3, or (at your option) any
-// later version.
-
-// This library is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// General Lesser Public License for more details.
-
-// You should have received a copy of the GNU Lesser General Public
-// License along with this program; see the file COPYING-LGPLV3.  If
-// not, see <http://www.gnu.org/licenses/>.
-//
 // Author: Dodji Seketeli
 
 #include <libgen.h>
@@ -57,8 +42,7 @@ emit_num_value(uint64_t value, const diff_context& ctxt, ostream& out)
     out << std::hex << std::showbase ;
   else
     out << std::dec;
-  out << value;
-  out << std::dec << std::noshowbase;
+  out << value << std::dec << std::noshowbase;
 }
 
 /// Convert a bits value into a byte value if the current diff context
@@ -254,7 +238,11 @@ represent_data_member(var_decl_sptr d,
       || (!get_member_is_static(d) && !get_data_member_is_laid_out(d)))
     return;
 
-  out << indent << "'" << d->get_pretty_representation() << "'";
+  out << indent
+      << "'"
+      << d->get_pretty_representation(/*internal=*/false,
+				      /*qualified_name=*/false)
+      << "'";
   if (!get_member_is_static(d))
     {
       // Do not emit offset information for data member of a union
@@ -420,7 +408,8 @@ represent(const var_diff_sptr	&diff,
   const uint64_t n_size = get_var_size_in_bits(n);
   const uint64_t o_offset = get_data_member_offset(o);
   const uint64_t n_offset = get_data_member_offset(n);
-  const string o_pretty_representation = o->get_pretty_representation();
+  const string o_pretty_representation =
+    o->get_pretty_representation(/*internal=*/false, /*qualified_name=*/false);
   // no n_pretty_representation here as it's only needed in a couple of places
   const bool show_size_offset_changes = ctxt->get_allowed_category()
 					& SIZE_OR_OFFSET_CHANGE_CATEGORY;
@@ -447,7 +436,9 @@ represent(const var_diff_sptr	&diff,
 
   if (is_strict_anonymous_data_member_change)
     {
-      const string n_pretty_representation = n->get_pretty_representation();
+      const string n_pretty_representation =
+	n->get_pretty_representation(/*internal=*/false,
+				     /*qualified_name=*/false);
       const type_base_sptr o_type = o->get_type(), n_type = n->get_type();
       if (o_pretty_representation != n_pretty_representation)
 	{
@@ -487,7 +478,9 @@ represent(const var_diff_sptr	&diff,
       ABG_ASSERT(o_anon != n_anon);
       // So we are looking at a non-anonymous data member change from
       // or to an anonymous data member.
-      const string n_pretty_representation = n->get_pretty_representation();
+      const string n_pretty_representation =
+	n->get_pretty_representation(/*internal=*/false,
+				     /*qualified_name=*/false);
       out << indent << (o_anon ? "anonymous " : "")
 	  << "data member " << o_pretty_representation;
       show_offset_or_size(" at offset", o_offset, *ctxt, out);
@@ -1194,6 +1187,21 @@ maybe_report_diff_for_symbol(const elf_symbol_sptr&	symbol1,
       reported = true;
     }
 
+  if (symbol1->get_crc() != 0 && symbol2->get_crc() != 0
+      && symbol1->get_crc() != symbol2->get_crc())
+    {
+      if (reported)
+	out << ",\n" << indent << "its CRC (modversions) changed from ";
+      else
+	out << "\n" << indent << "CRC value (modversions) changed from ";
+
+      out << std::showbase << std::hex
+	  << symbol1->get_crc() << " to " << symbol2->get_crc()
+	  << std::noshowbase << std::dec;
+
+      reported = true;
+    }
+
   if (reported)
     out << "\n";
 
