@@ -1,23 +1,8 @@
+// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 // -*- Mode: C++ -*-
 //
 // Copyright (C) 2013-2020 Red Hat, Inc.
 //
-// This file is part of the GNU Application Binary Interface Generic
-// Analysis and Instrumentation Library (libabigail).  This library is
-// free software; you can redistribute it and/or modify it under the
-// terms of the GNU Lesser General Public License as published by the
-// Free Software Foundation; either version 3, or (at your option) any
-// later version.
-
-// This library is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// General Lesser Public License for more details.
-
-// You should have received a copy of the GNU Lesser General Public
-// License along with this program; see the file COPYING-LGPLV3.  If
-// not, see <http://www.gnu.org/licenses/>.
-//
 // Author: Dodji Seketeli
 
 #ifndef __ABG_REPORTER_PRIV_H__
@@ -68,14 +53,14 @@
 	{								\
 	  string _name_ = _diff_->first_subject()->get_pretty_representation(); \
 	  if (_diff_->currently_reporting())				\
-	    out << indent << INTRO_TEXT << " '" << _name_ << "' changed; " \
-	      "details are being reported\n";				\
+	    out << indent << INTRO_TEXT << " '" << _name_		\
+		<< "' changed, as being reported\n";			\
 	  else								\
 	    {								\
 	      out << indent << INTRO_TEXT << " '"			\
-	      << _name_ << "' changed";				\
+	      << _name_ << "' changed";					\
 	      report_loc_info(D->first_subject(), *d.context(), out);	\
-	      out << ", as reported earlier\n";			\
+	      out << ", as reported earlier\n";				\
 	    }								\
 	  return ;							\
 	}								\
@@ -151,7 +136,8 @@ represent(const diff_context& ctxt,
 void
 represent_data_member(var_decl_sptr d,
 		      const diff_context_sptr& ctxt,
-		      ostream& out);
+		      ostream& out,
+		      const string& indent);
 
 void
 maybe_show_relative_offset_change(const var_diff_sptr &diff,
@@ -170,26 +156,24 @@ represent(const var_diff_sptr	&diff,
 	  const string&	indent = "",
 	  bool			local_only = false);
 
-bool
+void
 report_size_and_alignment_changes(type_or_decl_base_sptr	first,
 				  type_or_decl_base_sptr	second,
 				  diff_context_sptr		ctxt,
 				  ostream&			out,
-				  const string&		indent,
-				  bool				nl);
+				  const string&			indent);
 
 bool
 report_loc_info(const type_or_decl_base_sptr& tod,
 		const diff_context& ctxt,
 		ostream &out);
 
-bool
+void
 report_name_size_and_alignment_changes(decl_base_sptr		first,
 				       decl_base_sptr		second,
 				       diff_context_sptr	ctxt,
 				       ostream&		out,
-				       const string&		indent,
-				       bool			nl);
+				       const string&		indent);
 
 /// Represent the kind of difference we want report_mem_header() to
 /// report.
@@ -244,14 +228,18 @@ maybe_report_unreachable_type_changes(const corpus_diff& d,
 void
 maybe_report_interfaces_impacted_by_diff(const diff		*d,
 					 ostream		&out,
-					 const string		&indent,
-					 bool new_line_prefix = true);
+					 const string		&indent);
 
 void
 maybe_report_interfaces_impacted_by_diff(const diff_sptr	&d,
 					 ostream		&out,
-					 const string		&indent,
-					 bool new_line_prefix = false);
+					 const string		&indent);
+
+void
+maybe_report_data_members_replaced_by_anon_dm(const class_or_union_diff &d,
+					      ostream			&out,
+					      const string		indent);
+
 
 } // end namespace comparison
 } // end namespace abigail
