@@ -1,22 +1,7 @@
+// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2020 Red Hat, Inc.
-//
-// This file is part of the GNU Application Binary Interface Generic
-// Analysis and Instrumentation Library (libabigail).  This library is
-// free software; you can redistribute it and/or modify it under the
-// terms of the GNU Lesser General Public License as published by the
-// Free Software Foundation; either version 3, or (at your option) any
-// later version.
-
-// This library is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// General Lesser Public License for more details.
-
-// You should have received a copy of the GNU Lesser General Public
-// License along with this program; see the file COPYING-LGPLV3.  If
-// not, see <http://www.gnu.org/licenses/>.
+// Copyright (C) 2013-2021 Red Hat, Inc.
 
 ///@file
 
@@ -45,22 +30,21 @@
 #include <sys/time.h>
 #include <dirent.h>
 #include <time.h>
-#include <algorithm>
-#include <cstdlib>
-#include <cstring>
-#include <iterator>
 #include <ctype.h>
 #include <errno.h>
 #include <libgen.h>
 
+#include <algorithm>
+#include <cstdlib>
+#include <cstring>
 #include <fstream>
 #include <iostream>
+#include <iterator>
+#include <memory>
 #include <sstream>
 
-
 #include "abg-dwarf-reader.h"
 #include "abg-internal.h"
-#include "abg-cxx-compat.h"
 #include "abg-regex.h"
 
 // <headers defining libabigail's API go under here>
@@ -1349,9 +1333,6 @@ operator<<(ostream& output,
     case FILE_TYPE_XML_CORPUS_GROUP:
       repr = "native XML corpus group file type";
       break;
-    case FILE_TYPE_ZIP_CORPUS:
-      repr = "native ZIP corpus file type";
-      break;
     case FILE_TYPE_RPM:
       repr = "RPM file type";
       break;
@@ -1461,12 +1442,6 @@ guess_file_type(istream& in)
       && buf[11] == ' ')
     return FILE_TYPE_XML_CORPUS;
 
-  if (buf[0]    == 'P'
-      && buf[1] == 'K'
-      && buf[2] == 0x03
-      && buf[3] == 0x04)
-    return FILE_TYPE_ZIP_CORPUS;
-
   if ((unsigned char) buf[0]    == 0xed
       && (unsigned char) buf[1] == 0xab
       && (unsigned char) buf[2] == 0xee
@@ -1724,10 +1699,10 @@ struct malloced_char_star_deleter
 /// @param p the path to turn into an absolute path.
 ///
 /// @return a shared pointer to the resulting absolute path.
-abg_compat::shared_ptr<char>
+std::shared_ptr<char>
 make_path_absolute(const char*p)
 {
-  using abg_compat::shared_ptr;
+  using std::shared_ptr;
 
   shared_ptr<char> result;
 
@@ -2589,12 +2564,6 @@ build_corpus_group_from_kernel_dist_under(const string&	root,
 		      << t
 		      << "\n";
 
-	  // If we have been given a whitelist of functions and
-	  // variable symbols to look at, then we can avoid loading
-	  // and analyzing the ELF symbol table.
-	  bool do_ignore_symbol_table = !kabi_wl_paths.empty();
-	  set_ignore_symbol_table(*ctxt, do_ignore_symbol_table);
-
 	  group.reset(new corpus_group(env.get(), root));
 
 	  set_read_context_corpus_group(*ctxt, group);
@@ -2634,13 +2603,6 @@ build_corpus_group_from_kernel_dist_under(const string&	root,
 				 /*read_all_types=*/false,
 				 /*linux_kernel_mode=*/true);
 
-	      // If we have been given a whitelist of functions and
-	      // variable symbols to look at, then we can avoid loading
-	      // and analyzing the ELF symbol table.
-	      bool do_ignore_symbol_table = !kabi_wl_paths.empty();
-
-	      set_ignore_symbol_table(*ctxt, do_ignore_symbol_table);
-
 	      load_generate_apply_suppressions(*ctxt, suppr_paths,
 					       kabi_wl_paths, supprs);
 
