@@ -1,23 +1,8 @@
+// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 // -*- Mode: C++ -*-
 //
 // Copyright (C) 2013-2020 Red Hat, Inc.
 //
-// This file is part of the GNU Application Binary Interface Generic
-// Analysis and Instrumentation Library (libabigail).  This library is
-// free software; you can redistribute it and/or modify it under the
-// terms of the GNU Lesser General Public License as published by the
-// Free Software Foundation; either version 3, or (at your option) any
-// later version.
-
-// This library is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// General Lesser Public License for more details.
-
-// You should have received a copy of the GNU Lesser General Public
-// License along with this program; see the file COPYING-LGPLV3.  If
-// not, see <http://www.gnu.org/licenses/>.
-
 // Author: Dodji Seketeli
 
 /// @file
@@ -33,10 +18,11 @@
 
 #include <sys/wait.h>
 #include <cassert>
-#include <string>
+#include <cstdlib>
 #include <fstream>
 #include <iostream>
-#include <cstdlib>
+#include <memory>
+#include <string>
 #include "abg-workers.h"
 #include "abg-tools-utils.h"
 #include "test-utils.h"
@@ -787,6 +773,13 @@ InOutSpec in_out_specs[] =
    "data/test-diff-filter/test-PR26739-2-report-0.txt",
    "output/test-diff-filter/test-PR26739-2-report-0.txt",
   },
+  {
+   "data/test-diff-filter/test-PR26684-dwarf4.o",
+   "data/test-diff-filter/test-PR26684-dwarf5.o",
+   "--no-default-suppression --leaf-changes-only",
+   "data/test-diff-filter/test-PR26684-report-0.txt",
+   "output/test-diff-filter/test-PR26684-report-0.txt",
+  },
   {
    "data/test-diff-filter/test-PR27331-v0.o",
    "data/test-diff-filter/test-PR27331-v1.o",
@@ -794,6 +787,27 @@ InOutSpec in_out_specs[] =
    "data/test-diff-filter/test-PR27331-report-0.txt",
    "output/test-diff-filter/test-PR27331-report-0.txt",
   },
+  {
+   "data/test-diff-filter/test-PR27569-v0.abi",
+   "data/test-diff-filter/test-PR27569-v1.abi",
+   "--no-default-suppression",
+   "data/test-diff-filter/test-PR27569-report-0.txt",
+   "output/test-diff-filter/test-PR27569-report-0.txt",
+  },
+  {
+   "data/test-diff-filter/test-PR27598-v0.o",
+   "data/test-diff-filter/test-PR27598-v0.o",
+   "--no-default-suppression",
+   "data/test-diff-filter/test-PR27598-report-0.txt",
+   "output/test-diff-filter/test-PR27598-report-0.txt",
+  },
+  {
+   "data/test-diff-filter/test-PR27995.abi",
+   "data/test-diff-filter/test-PR27995.abi",
+   "--no-default-suppression",
+   "data/test-diff-filter/test-PR27995-report-0.txt",
+   "output/test-diff-filter/test-PR27995-report-0.txt",
+  },
   // This should be the last entry
   {NULL, NULL, NULL, NULL, NULL}
 };
@@ -891,7 +905,7 @@ int
 main()
 {
   using std::vector;
-  using abg_compat::dynamic_pointer_cast;
+  using std::dynamic_pointer_cast;
   using abigail::workers::queue;
   using abigail::workers::task;
   using abigail::workers::task_sptr;
