@@ -1,23 +1,8 @@
+// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 // -*- Mode: C++ -*-
 //
 // Copyright (C) 2013-2020 Red Hat, Inc.
 //
-// This file is part of the GNU Application Binary Interface Generic
-// Analysis and Instrumentation Library (libabigail).  This library is
-// free software; you can redistribute it and/or modify it under the
-// terms of the GNU Lesser General Public License as published by the
-// Free Software Foundation; either version 3, or (at your option) any
-// later version.
-
-// This library is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// General Lesser Public License for more details.
-
-// You should have received a copy of the GNU Lesser General Public
-// License along with this program; see the file COPYING-LGPLV3.  If
-// not, see <http://www.gnu.org/licenses/>.
-
 // Author: Sinny Kumari
 
 /// @file
@@ -35,10 +20,11 @@
 #include "config.h"
 #include <sys/wait.h>
 #include <cassert>
-#include <cstring>
-#include <string>
 #include <cstdlib>
+#include <cstring>
 #include <iostream>
+#include <memory>
+#include <string>
 #include <vector>
 #include "abg-workers.h"
 #include "test-utils.h"
@@ -684,6 +670,69 @@ static InOutSpec in_out_specs[] =
     "data/test-diff-pkg/sshpass-1.07-1.fc34.x86_64-self-check-report-0.txt",
     "output/test-diff-pkg/sshpass-1.07-1.fc34.x86_64-self-check-report-0.txt"
   },
+  {
+    "data/test-diff-pkg/nmap-7.70-5.el8_testjcc.x86_64.rpm",
+    "data/test-diff-pkg/nmap-7.70-5.el8_testjcc.x86_64.rpm",
+    "--self-check",
+    "",
+    "data/test-diff-pkg/nmap-debuginfo-7.70-5.el8_testjcc.x86_64.rpm",
+    "data/test-diff-pkg/nmap-debuginfo-7.70-5.el8_testjcc.x86_64.rpm",
+    "",
+    "",
+    "data/test-diff-pkg/nmap-7.70-5.el8_testjcc.x86_64-self-check-report-0.txt",
+    "output/test-diff-pkg/nmap-7.70-5.el8_testjcc.x86_64-self-check-report-0.txt"
+  } ,
+  {
+    "data/test-diff-pkg/elfutils-libs-0.183-1.el9.x86_64.rpm",
+    "data/test-diff-pkg/elfutils-libs-0.183-1.el9.x86_64.rpm",
+    "--self-check",
+    "",
+    "data/test-diff-pkg/elfutils-debuginfo-0.183-1.el9.x86_64.rpm,"
+    "data/test-diff-pkg/elfutils-libs-debuginfo-0.183-1.el9.x86_64.rpm",
+    "data/test-diff-pkg/elfutils-debuginfo-0.183-1.el9.x86_64.rpm,"
+    "data/test-diff-pkg/elfutils-libs-debuginfo-0.183-1.el9.x86_64.rpm",
+    "",
+    "",
+    "data/test-diff-pkg/elfutils-libs-debuginfo-0.183-1.el9.x86_64-self-check-report-0.txt",
+    "output/test-diff-pkg/elfutils-libs-debuginfo-0.183-1.el9.x86_64-self-check-report-0.txt"
+  } ,
+  {
+    "data/test-diff-pkg/graphviz-2.44.0-18.el9.aarch64.rpm",
+    "data/test-diff-pkg/graphviz-2.44.0-18.el9.aarch64.rpm",
+    "--self-check",
+    "",
+    "data/test-diff-pkg/graphviz-debuginfo-2.44.0-18.el9.aarch64.rpm",
+    "data/test-diff-pkg/graphviz-debuginfo-2.44.0-18.el9.aarch64.rpm",
+    "",
+    "",
+    "data/test-diff-pkg/graphviz-2.44.0-18.el9.aarch64-self-check-report-0.txt",
+    "output/test-diff-pkg/graphviz-2.44.0-18.el9.aarch64-self-check-report-0.txt"
+  } ,
+  {
+    "data/test-diff-pkg/libxcrypt-4.1.1-6.el8.x86_64.rpm",
+    "data/test-diff-pkg/libxcrypt-4.4.18-3.el9.x86_64.rpm",
+    "",
+    "",
+    "data/test-diff-pkg/libxcrypt-debuginfo-4.1.1-6.el8.x86_64.rpm",
+    "data/test-diff-pkg/libxcrypt-debuginfo-4.4.18-3.el9.x86_64.rpm",
+    "",
+    "",
+    "data/test-diff-pkg/libxcrypt-4.1.1-6.el8.x86_64--libxcrypt-4.1.1-6.el8.x86_64-output-1.txt",
+    "output/test-diff-pkg/libxcrypt-4.1.1-6.el8.x86_64--libxcrypt-4.1.1-6.el8.x86_64-output-1.txt"
+  } ,
+  {
+    "data/test-diff-pkg/libxcrypt-4.1.1-6.el8.x86_64.rpm",
+    "data/test-diff-pkg/libxcrypt-compat-4.4.18-3.el9.x86_64.rpm",
+    "",
+    "",
+    "data/test-diff-pkg/libxcrypt-debuginfo-4.1.1-6.el8.x86_64.rpm",
+    "data/test-diff-pkg/libxcrypt-debuginfo-4.4.18-3.el9.x86_64.rpm,"
+    "data/test-diff-pkg/libxcrypt-compat-debuginfo-4.4.18-3.el9.x86_64.rpm",
+    "",
+    "",
+    "data/test-diff-pkg/libxcrypt-4.1.1-6.el8.x86_64--libxcrypt-compat-4.4.18-3.el9.x86_64-report-1.txt",
+    "output/test-diff-pkg/libxcrypt-4.1.1-6.el8.x86_64--libxcrypt-compat-4.4.18-3.el9.x86_64-report-1.txt"
+  } ,
 #endif // WITH_RPM_4_15
 #endif //WITH_RPM
 
@@ -866,7 +915,7 @@ int
 main()
 {
   using std::vector;
-  using abg_compat::dynamic_pointer_cast;
+  using std::dynamic_pointer_cast;
   using abigail::workers::queue;
   using abigail::workers::task;
   using abigail::workers::task_sptr;
