@@ -1,23 +1,8 @@
+// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 // -*- Mode: C++ -*-
 //
 // Copyright (C) 2013-2020 Red Hat, Inc.
 //
-// This file is part of the GNU Application Binary Interface Generic
-// Analysis and Instrumentation Library (libabigail).  This library is
-// free software; you can redistribute it and/or modify it under the
-// terms of the GNU Lesser General Public License as published by the
-// Free Software Foundation; either version 3, or (at your option) any
-// later version.
-
-// This library is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// General Lesser Public License for more details.
-
-// You should have received a copy of the GNU Lesser General Public
-// License along with this program; see the file COPYING-LGPLV3.  If
-// not, see <http://www.gnu.org/licenses/>.
-
 // Author: Dodji Seketeli
 
 /// @file
@@ -31,12 +16,13 @@
 /// one.
 
 #include <sys/wait.h>
+#include <cstdlib>
 #include <cstring>
-#include <string>
 #include <fstream>
 #include <iostream>
-#include <cstdlib>
-#include "abg-cxx-compat.h"
+#include <memory>
+#include <string>
+#include <unordered_set>
 #include "abg-tools-utils.h"
 #include "abg-workers.h"
 #include "test-utils.h"
@@ -2180,7 +2166,7 @@ main(int argc, char *argv[])
   const std::string out_base_path = std::string(get_build_dir()) + "/tests/";
 
   // output paths need to be unique to avoid collisions during parallel testing
-  abg_compat::unordered_set<std::string> out_paths;
+  std::unordered_set<std::string> out_paths;
   bool non_unique_output_paths = false;
   for (InOutSpec* s = in_out_specs; s->in_elfv0_path; ++s) {
 	if (!out_paths.insert(s->out_report_path).second) {
@@ -2194,7 +2180,7 @@ main(int argc, char *argv[])
 
   for (InOutSpec* s = in_out_specs; s->in_elfv0_path; ++s)
     {
-      abg_compat::shared_ptr<test_task> t(
+      std::shared_ptr<test_task> t(
 	  new test_task(*s, in_base_path, out_base_path));
       ABG_ASSERT(task_queue.schedule_task(t));
     }
@@ -2214,8 +2200,8 @@ main(int argc, char *argv[])
        ti != completed_tasks.end();
        ++ti)
     {
-      abg_compat::shared_ptr<test_task> t =
-	  abg_compat::dynamic_pointer_cast<test_task>(*ti);
+      std::shared_ptr<test_task> t =
+	  std::dynamic_pointer_cast<test_task>(*ti);
       if (!t->is_ok)
 	{
 	  is_ok = false;
