@@ -1,34 +1,20 @@
+// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 // -*- Mode: C++ -*-
 //
 // Copyright (C) 2013-2020 Red Hat, Inc.
 //
-// This file is part of the GNU Application Binary Interface Generic
-// Analysis and Instrumentation Library (libabigail).  This library is
-// free software; you can redistribute it and/or modify it under the
-// terms of the GNU Lesser General Public License as published by the
-// Free Software Foundation; either version 3, or (at your option) any
-// later version.
-
-// This library is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// General Lesser Public License for more details.
-
-// You should have received a copy of the GNU Lesser General Public
-// License along with this program; see the file COPYING-LGPLV3.  If
-// not, see <http://www.gnu.org/licenses/>.
-
 // Author: Dodji Seketeli
 
 /// @file read ELF binaries containing DWARF, save them in XML corpus
 /// files and diff the corpus files against reference XML corpus
 /// files.
 
-#include <string>
+#include <cstdlib>
 #include <fstream>
 #include <iostream>
+#include <memory>
+#include <string>
 #include <vector>
-#include <cstdlib>
 #include "abg-ir.h"
 #include "abg-dwarf-reader.h"
 #include "abg-workers.h"
@@ -40,7 +26,7 @@ using std::vector;
 using std::string;
 using std::ofstream;
 using std::cerr;
-using abg_compat::dynamic_pointer_cast;
+using std::dynamic_pointer_cast;
 using abigail::tests::get_build_dir;
 using abigail::dwarf_reader::read_corpus_from_elf;
 using abigail::dwarf_reader::read_context;
@@ -60,6 +46,7 @@ struct InOutSpec
 {
   const char* in_elf_path;
   const char* in_suppr_spec_path;
+  const char* in_public_headers_path;
   type_id_style_kind type_id_style;
   const char* in_abi_path;
   const char* out_abi_path;
@@ -71,6 +58,7 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/test0",
     "",
+    "",
     SEQUENCE_TYPE_ID_STYLE,
     "data/test-read-dwarf/test0.abi",
     "output/test-read-dwarf/test0.abi"
@@ -78,6 +66,7 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/test0",
     "",
+    "",
     HASH_TYPE_ID_STYLE,
     "data/test-read-dwarf/test0.hash.abi",
     "output/test-read-dwarf/test0.hash.abi"
@@ -85,6 +74,7 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/test1",
     "",
+    "",
     SEQUENCE_TYPE_ID_STYLE,
     "data/test-read-dwarf/test1.abi",
     "output/test-read-dwarf/test1.abi"
@@ -92,6 +82,7 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/test1",
     "",
+    "",
     HASH_TYPE_ID_STYLE,
     "data/test-read-dwarf/test1.hash.abi",
     "output/test-read-dwarf/test1.hash.abi"
@@ -99,6 +90,7 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/test2.so",
     "",
+    "",
     SEQUENCE_TYPE_ID_STYLE,
     "data/test-read-dwarf/test2.so.abi",
     "output/test-read-dwarf/test2.so.abi"
@@ -106,6 +98,7 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/test2.so",
     "",
+    "",
     HASH_TYPE_ID_STYLE,
     "data/test-read-dwarf/test2.so.hash.abi",
     "output/test-read-dwarf/test2.so.hash.abi"
@@ -113,6 +106,7 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/test3.so",
     "",
+    "",
     SEQUENCE_TYPE_ID_STYLE,
     "data/test-read-dwarf/test3.so.abi",
     "output/test-read-dwarf/test3.so.abi"
@@ -120,13 +114,60 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/test3.so",
     "",
+    "",
     HASH_TYPE_ID_STYLE,
     "data/test-read-dwarf/test3.so.hash.abi",
     "output/test-read-dwarf/test3.so.hash.abi"
   },
+  // suppress all except the main symbol of a group of aliases
+  {
+    "data/test-read-dwarf/test3.so",
+    "data/test-read-dwarf/test3-alias-1.suppr",
+    "",
+    HASH_TYPE_ID_STYLE,
+    "data/test-read-dwarf/test3-alias-1.so.hash.abi",
+    "output/test-read-dwarf/test3-alias-1.so.hash.abi"
+  },
+  // suppress the main symbol of a group of aliases
+  {
+    "data/test-read-dwarf/test3.so",
+    "data/test-read-dwarf/test3-alias-2.suppr",
+    "",
+    HASH_TYPE_ID_STYLE,
+    "data/test-read-dwarf/test3-alias-2.so.hash.abi",
+    "output/test-read-dwarf/test3-alias-2.so.hash.abi"
+  },
+  // suppress all except one non main symbol of a group of aliases
+  {
+    "data/test-read-dwarf/test3.so",
+    "data/test-read-dwarf/test3-alias-3.suppr",
+    "",
+    HASH_TYPE_ID_STYLE,
+    "data/test-read-dwarf/test3-alias-3.so.hash.abi",
+    "output/test-read-dwarf/test3-alias-3.so.hash.abi"
+  },
+  // suppress all symbols of a group of aliases
+  {
+    "data/test-read-dwarf/test3.so",
+    "data/test-read-dwarf/test3-alias-4.suppr",
+    "",
+    HASH_TYPE_ID_STYLE,
+    "data/test-read-dwarf/test3-alias-4.so.hash.abi",
+    "output/test-read-dwarf/test3-alias-4.so.hash.abi"
+  },
+  // suppress the main symbols with alias (function+variable) in .o file
+  {
+    "data/test-read-dwarf/test-suppressed-alias.o",
+    "data/test-read-dwarf/test-suppressed-alias.suppr",
+    "",
+    HASH_TYPE_ID_STYLE,
+    "data/test-read-dwarf/test-suppressed-alias.o.abi",
+    "output/test-read-dwarf/test-suppressed-alias.o.abi",
+  },
   {
     "data/test-read-dwarf/test4.so",
     "",
+    "",
     SEQUENCE_TYPE_ID_STYLE,
     "data/test-read-dwarf/test4.so.abi",
     "output/test-read-dwarf/test4.so.abi"
@@ -134,6 +175,7 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/test4.so",
     "",
+    "",
     HASH_TYPE_ID_STYLE,
     "data/test-read-dwarf/test4.so.hash.abi",
     "output/test-read-dwarf/test4.so.hash.abi"
@@ -141,6 +183,7 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/test5.o",
     "",
+    "",
     SEQUENCE_TYPE_ID_STYLE,
     "data/test-read-dwarf/test5.o.abi",
     "output/test-read-dwarf/test5.o.abi"
@@ -148,6 +191,7 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/test5.o",
     "",
+    "",
     HASH_TYPE_ID_STYLE,
     "data/test-read-dwarf/test5.o.hash.abi",
     "output/test-read-dwarf/test5.o.hash.abi"
@@ -155,6 +199,7 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/test6.so",
     "",
+    "",
     SEQUENCE_TYPE_ID_STYLE,
     "data/test-read-dwarf/test6.so.abi",
     "output/test-read-dwarf/test6.so.abi"
@@ -162,6 +207,7 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/test6.so",
     "",
+    "",
     HASH_TYPE_ID_STYLE,
     "data/test-read-dwarf/test6.so.hash.abi",
     "output/test-read-dwarf/test6.so.hash.abi"
@@ -169,6 +215,7 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/test7.so",
     "",
+    "",
     SEQUENCE_TYPE_ID_STYLE,
     "data/test-read-dwarf/test7.so.abi",
     "output/test-read-dwarf/test7.so.abi"
@@ -176,6 +223,7 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/test7.so",
     "",
+    "",
     HASH_TYPE_ID_STYLE,
     "data/test-read-dwarf/test7.so.hash.abi",
     "output/test-read-dwarf/test7.so.hash.abi"
@@ -183,6 +231,7 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/test8-qualified-this-pointer.so",
     "",
+    "",
     SEQUENCE_TYPE_ID_STYLE,
     "data/test-read-dwarf/test8-qualified-this-pointer.so.abi",
     "output/test-read-dwarf/test8-qualified-this-pointer.so.abi"
@@ -190,6 +239,7 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/test8-qualified-this-pointer.so",
     "",
+    "",
     HASH_TYPE_ID_STYLE,
     "data/test-read-dwarf/test8-qualified-this-pointer.so.hash.abi",
     "output/test-read-dwarf/test8-qualified-this-pointer.so.hash.abi"
@@ -197,6 +247,7 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/test9-pr18818-clang.so",
     "",
+    "",
     SEQUENCE_TYPE_ID_STYLE,
     "data/test-read-dwarf/test9-pr18818-clang.so.abi",
     "output/test-read-dwarf/test9-pr18818-clang.so.abi"
@@ -204,6 +255,7 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/test10-pr18818-gcc.so",
     "",
+    "",
     SEQUENCE_TYPE_ID_STYLE,
     "data/test-read-dwarf/test10-pr18818-gcc.so.abi",
     "output/test-read-dwarf/test10-pr18818-gcc.so.abi"
@@ -211,6 +263,7 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/test11-pr18828.so",
     "",
+    "",
     SEQUENCE_TYPE_ID_STYLE,
     "data/test-read-dwarf/test11-pr18828.so.abi",
     "output/test-read-dwarf/test11-pr18828.so.abi",
@@ -218,6 +271,7 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/test12-pr18844.so",
     "",
+    "",
     SEQUENCE_TYPE_ID_STYLE,
     "data/test-read-dwarf/test12-pr18844.so.abi",
     "output/test-read-dwarf/test12-pr18844.so.abi",
@@ -225,6 +279,7 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/test13-pr18894.so",
     "",
+    "",
     SEQUENCE_TYPE_ID_STYLE,
     "data/test-read-dwarf/test13-pr18894.so.abi",
     "output/test-read-dwarf/test13-pr18894.so.abi",
@@ -232,6 +287,7 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/test14-pr18893.so",
     "",
+    "",
     SEQUENCE_TYPE_ID_STYLE,
     "data/test-read-dwarf/test14-pr18893.so.abi",
     "output/test-read-dwarf/test14-pr18893.so.abi",
@@ -239,6 +295,7 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/test15-pr18892.so",
     "",
+    "",
     SEQUENCE_TYPE_ID_STYLE,
     "data/test-read-dwarf/test15-pr18892.so.abi",
     "output/test-read-dwarf/test15-pr18892.so.abi",
@@ -246,6 +303,7 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/test16-pr18904.so",
     "",
+    "",
     SEQUENCE_TYPE_ID_STYLE,
     "data/test-read-dwarf/test16-pr18904.so.abi",
     "output/test-read-dwarf/test16-pr18904.so.abi",
@@ -253,6 +311,7 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/test17-pr19027.so",
     "",
+    "",
     SEQUENCE_TYPE_ID_STYLE,
     "data/test-read-dwarf/test17-pr19027.so.abi",
     "output/test-read-dwarf/test17-pr19027.so.abi",
@@ -260,6 +319,7 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/test18-pr19037-libvtkRenderingLIC-6.1.so",
     "",
+    "",
     SEQUENCE_TYPE_ID_STYLE,
     "data/test-read-dwarf/test18-pr19037-libvtkRenderingLIC-6.1.so.abi",
     "output/test-read-dwarf/test18-pr19037-libvtkRenderingLIC-6.1.so.abi",
@@ -267,6 +327,7 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/test19-pr19023-libtcmalloc_and_profiler.so",
     "",
+    "",
     SEQUENCE_TYPE_ID_STYLE,
     "data/test-read-dwarf/test19-pr19023-libtcmalloc_and_profiler.so.abi",
     "output/test-read-dwarf/test19-pr19023-libtcmalloc_and_profiler.so.abi",
@@ -274,6 +335,7 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/test20-pr19025-libvtkParallelCore-6.1.so",
     "",
+    "",
     SEQUENCE_TYPE_ID_STYLE,
     "data/test-read-dwarf/test20-pr19025-libvtkParallelCore-6.1.so.abi",
     "output/test-read-dwarf/test20-pr19025-libvtkParallelCore-6.1.so.abi",
@@ -281,6 +343,7 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/test21-pr19092.so",
     "",
+    "",
     SEQUENCE_TYPE_ID_STYLE,
     "data/test-read-dwarf/test21-pr19092.so.abi",
     "output/test-read-dwarf/test21-pr19092.so.abi",
@@ -288,6 +351,7 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/test22-pr19097-libstdc++.so.6.0.17.so",
     "",
+    "",
     SEQUENCE_TYPE_ID_STYLE,
     "data/test-read-dwarf/test22-pr19097-libstdc++.so.6.0.17.so.abi",
     "output/test-read-dwarf/test22-pr19097-libstdc++.so.6.0.17.so.abi",
@@ -295,6 +359,7 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/libtest23.so",
     "",
+    "",
     SEQUENCE_TYPE_ID_STYLE,
     "data/test-read-dwarf/libtest23.so.abi",
     "output/test-read-dwarf/libtest23.so.abi",
@@ -302,6 +367,7 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/libtest24-drop-fns.so",
     "data/test-read-dwarf/test24-drop-fns-0.suppr",
+    "",
     SEQUENCE_TYPE_ID_STYLE,
     "data/test-read-dwarf/libtest24-drop-fns.so.abi",
     "output/test-read-dwarf/libtest24-drop-fns.so.abi",
@@ -309,6 +375,7 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/libtest24-drop-fns.so",
     "",
+    "",
     SEQUENCE_TYPE_ID_STYLE,
     "data/test-read-dwarf/libtest24-drop-fns-2.so.abi",
     "output/test-read-dwarf/libtest24-drop-fns-2.so.abi",
@@ -316,6 +383,7 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/PR22015-libboost_iostreams.so",
     "",
+    "",
     SEQUENCE_TYPE_ID_STYLE,
     "data/test-read-dwarf/PR22015-libboost_iostreams.so.abi",
     "output/test-read-dwarf/PR22015-libboost_iostreams.so.abi",
@@ -323,6 +391,7 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/PR22122-libftdc.so",
     "",
+    "",
     SEQUENCE_TYPE_ID_STYLE,
     "data/test-read-dwarf/PR22122-libftdc.so.abi",
     "output/test-read-dwarf/PR22122-libftdc.so.abi",
@@ -330,6 +399,7 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/PR24378-fn-is-not-scope.o",
     "",
+    "",
     SEQUENCE_TYPE_ID_STYLE,
     "data/test-read-dwarf/PR24378-fn-is-not-scope.abi",
     "output/test-read-dwarf/PR24378-fn-is-not-scope.abi",
@@ -338,6 +408,7 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/PR25007-sdhci.ko",
     "",
+    "",
     SEQUENCE_TYPE_ID_STYLE,
     "data/test-read-dwarf/PR25007-sdhci.ko.abi",
     "output/test-read-dwarf/PR25007-sdhci.ko.abi",
@@ -347,6 +418,7 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/PR25042-libgdbm-clang-dwarf5.so.6.0.0",
     "",
+    "",
     SEQUENCE_TYPE_ID_STYLE,
     "data/test-read-dwarf/PR25042-libgdbm-clang-dwarf5.so.6.0.0.abi",
     "output/test-read-dwarf/PR25042-libgdbm-clang-dwarf5.so.6.0.0.abi",
@@ -355,6 +427,7 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/test25-bogus-binary.elf",
     "",
+    "",
     SEQUENCE_TYPE_ID_STYLE,
     NULL,
     NULL,
@@ -362,6 +435,7 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/test26-bogus-binary.elf",
     "",
+    "",
     SEQUENCE_TYPE_ID_STYLE,
     NULL,
     NULL,
@@ -369,6 +443,7 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/test27-bogus-binary.elf",
     "",
+    "",
     SEQUENCE_TYPE_ID_STYLE,
     NULL,
     NULL,
@@ -376,6 +451,7 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/PR26261/PR26261-exe",
     "",
+    "",
     SEQUENCE_TYPE_ID_STYLE,
     "data/test-read-dwarf/PR26261/PR26261-exe.abi",
     "output/test-read-dwarf/PR26261/PR26261-exe.abi",
@@ -383,6 +459,7 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/test-PR26568-1.o",
     "",
+    "",
     SEQUENCE_TYPE_ID_STYLE,
     "data/test-read-dwarf/test-PR26568-1.o.abi",
     "output/test-read-dwarf/test-PR26568-1.o.abi",
@@ -390,6 +467,7 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/test-PR26568-2.o",
     "",
+    "",
     SEQUENCE_TYPE_ID_STYLE,
     "data/test-read-dwarf/test-PR26568-2.o.abi",
     "output/test-read-dwarf/test-PR26568-2.o.abi",
@@ -397,12 +475,29 @@ InOutSpec in_out_specs[] =
   {
     "data/test-read-dwarf/test-libandroid.so",
     "",
+    "",
     HASH_TYPE_ID_STYLE,
     "data/test-read-dwarf/test-libandroid.so.abi",
     "output/test-read-dwarf/test-libandroid.so.abi",
   },
+  {
+    "data/test-read-dwarf/PR27700/test-PR27700.o",
+    "",
+    "data/test-read-dwarf/PR27700/pub-incdir",
+    HASH_TYPE_ID_STYLE,
+    "data/test-read-dwarf/PR27700/test-PR27700.abi",
+    "output/test-read-dwarf/PR27700/test-PR27700.abi",
+  },
+  {
+    "data/test-read-dwarf/test-libaaudio.so",
+    "",
+    "",
+    HASH_TYPE_ID_STYLE,
+    "data/test-read-dwarf/test-libaaudio.so.abi",
+    "output/test-read-dwarf/test-libaaudio.so.abi",
+  },
   // This should be the last entry.
-  {NULL, NULL, SEQUENCE_TYPE_ID_STYLE, NULL, NULL}
+  {NULL, NULL, NULL, SEQUENCE_TYPE_ID_STYLE, NULL, NULL}
 };
 
 using abigail::suppr::suppression_sptr;
@@ -422,6 +517,33 @@ set_suppressions(read_context& read_ctxt, const string& path)
   add_read_context_suppressions(read_ctxt, supprs);
 }
 
+/// Define what headers contain public types definitions.
+///
+/// This automatically generates suppression specifications from the
+/// set of header files present under a given directory.  Those
+/// specifications actually suppress types that are *not* defined in
+/// the headers found at a given directory.
+///
+/// @param read_ctxt the context used to read the ELF binary.
+///
+/// @param path the path to a directory where header files are to be
+/// found.
+static void
+set_suppressions_from_headers(read_context& read_ctxt, const string& path)
+{
+  vector<string> files;
+  suppression_sptr suppr =
+    abigail::tools_utils::gen_suppr_spec_from_headers(path, files);
+
+  if (suppr)
+    {
+      suppr->set_drops_artifact_from_ir(true);
+      suppressions_type supprs;
+      supprs.push_back(suppr);
+      add_read_context_suppressions(read_ctxt, supprs);
+    }
+}
+
 /// The task that peforms the tests.
 struct test_task : public abigail::workers::task
 {
@@ -451,7 +573,8 @@ struct test_task : public abigail::workers::task
   virtual void
   perform()
   {
-    string in_elf_path, in_abi_path, in_suppr_spec_path, out_abi_path;
+    string in_elf_path, in_abi_path, in_suppr_spec_path, in_public_headers_path,
+      out_abi_path;
     abigail::ir::environment_sptr env;
 
     in_elf_path = in_elf_base + spec.in_elf_path;
@@ -460,6 +583,11 @@ struct test_task : public abigail::workers::task
     else
       in_suppr_spec_path.clear();
 
+    if (spec.in_public_headers_path)
+      in_public_headers_path = spec.in_public_headers_path;
+    if (!in_public_headers_path.empty())
+      in_public_headers_path = in_elf_base + spec.in_public_headers_path;
+
     env.reset(new abigail::ir::environment);
     abigail::dwarf_reader::status status =
     abigail::dwarf_reader::STATUS_UNKNOWN;
@@ -472,6 +600,9 @@ struct test_task : public abigail::workers::task
     if (!in_suppr_spec_path.empty())
       set_suppressions(*ctxt, in_suppr_spec_path);
 
+    if (!in_public_headers_path.empty())
+      set_suppressions_from_headers(*ctxt, in_public_headers_path);
+
     abigail::corpus_sptr corp = read_corpus_from_elf(*ctxt, status);
     // if there is no output and no input, assume that we do not care about the
     // actual read result, just that it succeeded.
@@ -515,7 +646,11 @@ struct test_task : public abigail::workers::task
     of.close();
 
     string abidw = string(get_build_dir()) + "/tools/abidw";
-    string cmd = abidw + " --abidiff " + in_elf_path;
+    string drop_private_types;
+    if (!in_public_headers_path.empty())
+      drop_private_types += "--headers-dir " + in_public_headers_path +
+	" --drop-private-types";
+    string cmd = abidw + " " + drop_private_types + " --abidiff " + in_elf_path;
     if (system(cmd.c_str()))
       {
 	error_message = string("ABIs differ:\n")
