@@ -1,23 +1,8 @@
+// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 // -*- Mode: C++ -*-
 //
 // Copyright (C) 2013-2020 Red Hat, Inc.
 //
-// This file is part of the GNU Application Binary Interface Generic
-// Analysis and Instrumentation Library (libabigail).  This library is
-// free software; you can redistribute it and/or modify it under the
-// terms of the GNU Lesser General Public License as published by the
-// Free Software Foundation; either version 3, or (at your option) any
-// later version.
-
-// This library is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// General Lesser Public License for more details.
-
-// You should have received a copy of the GNU Lesser General Public
-// License along with this program; see the file COPYING-LGPLV3.  If
-// not, see <http://www.gnu.org/licenses/>.
-
 // Author: Dodji Seketeli
 
 /// @file
@@ -36,10 +21,11 @@
 ///
 /// The comparison is expected to yield the empty set.
 
-#include <string>
+#include <cstdlib>
 #include <fstream>
 #include <iostream>
-#include <cstdlib>
+#include <memory>
+#include <string>
 #include "abg-tools-utils.h"
 #include "test-utils.h"
 #include "abg-dwarf-reader.h"
@@ -63,6 +49,8 @@ const char* elf_paths[] =
   "data/test-types-stability/pr19142-topo.o",
   "data/test-types-stability/pr19204-libtcmalloc.so.4.2.6-xlc",
   "data/test-types-stability/PR27165-libzmq.so.5.2.3",
+  "data/test-types-stability/pr27980-libc.so",
+  "data/test-types-stability/PR27086-libstdc++.so.6.0.26",
   // The below should always be the last element of array.
   0
 };
@@ -119,7 +107,7 @@ int
 main()
 {
   using std::vector;
-  using abg_compat::dynamic_pointer_cast;
+  using std::dynamic_pointer_cast;
   using abigail::workers::queue;
   using abigail::workers::task;
   using abigail::workers::task_sptr;
