@@ -1,31 +1,18 @@
+// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2020 Red Hat, Inc.
-//
-// This file is part of the GNU Application Binary Interface Generic
-// Analysis and Instrumentation Library (libabigail).  This library is
-// free software; you can redistribute it and/or modify it under the
-// terms of the GNU Lesser General Public License as published by the
-// Free Software Foundation; either version 3, or (at your option) any
-// later version.
-
-// This library is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// General Lesser Public License for more details.
-
-// You should have received a copy of the GNU Lesser General Public
-// License along with this program; see the file COPYING-LGPLV3.  If
-// not, see <http://www.gnu.org/licenses/>.
+// Copyright (C) 2013-2021 Red Hat, Inc.
 //
 // Author: Dodji Seketeli
 
 /// @file
 
+#include "config.h"
 #include <cstring>
-#include <vector>
-#include <string>
 #include <iostream>
+#include <memory>
+#include <string>
+#include <vector>
 #include "abg-config.h"
 #include "abg-comp-filter.h"
 #include "abg-suppression.h"
@@ -38,7 +25,7 @@ using std::string;
 using std::ostream;
 using std::cout;
 using std::cerr;
-using abg_compat::shared_ptr;
+using std::shared_ptr;
 using abigail::ir::environment;
 using abigail::ir::environment_sptr;
 using abigail::translation_unit;
@@ -78,8 +65,10 @@ struct options
   vector<string>	drop_var_regex_patterns;
   vector<string>	keep_fn_regex_patterns;
   vector<string>	keep_var_regex_patterns;
-  string		headers_dir1;
-  string		headers_dir2;
+  vector<string>	headers_dirs1;
+  vector<string>        header_files1;
+  vector<string>	headers_dirs2;
+  vector<string>        header_files2;
   bool			drop_private_types;
   bool			linux_kernel_mode;
   bool			no_default_supprs;
@@ -112,6 +101,9 @@ struct options
   bool			dump_diff_tree;
   bool			show_stats;
   bool			do_log;
+#ifdef WITH_DEBUG_SELF_COMPARISON
+  bool			do_debug;
+#endif
   vector<char*> di_root_paths1;
   vector<char*> di_root_paths2;
   vector<char**> prepared_di_root_paths1;
@@ -153,6 +145,10 @@ struct options
       dump_diff_tree(),
       show_stats(),
       do_log()
+#ifdef WITH_DEBUG_SELF_COMPARISON
+    ,
+    do_debug()
+#endif
   {}
 
   ~options()
@@ -183,7 +179,9 @@ display_usage(const string& prog_name, ostream& out)
     << " --debug-info-dir1|--d1 <path> the root for the debug info of file1\n"
     << " --debug-info-dir2|--d2 <path> the root for the debug info of file2\n"
     << " --headers-dir1|--hd1 <path>  the path to headers of file1\n"
+    << " --header-file1|--hf1 <path>  the path to one header of file1\n"
     << " --headers-dir2|--hd2 <path>  the path to headers of file2\n"
+    << " --header-file2|--hf2 <path>  the path to one header of file2\n"
     << " --drop-private-types  drop private types from "
     "internal representation\n"
     << " --no-linux-kernel-mode  don't consider the input binaries as "
@@ -198,7 +196,7 @@ display_usage(const string& prog_name, ostream& out)
     << " --no-corpus-path  do not take the path to the corpora into account\n"
     << " --fail-no-debug-info  bail out if no debug info was found\n"
     << " --leaf-changes-only|-l  only show leaf changes, "
-    "so no change impact analysis\n"
+    "so no change impact analysis (implies --redundant)\n"
     << " --deleted-fns  display deleted public functions\n"
     << " --changed-fns  display changed public functions\n"
     << " --added-fns  display added public functions\n"
@@ -231,11 +229,13 @@ display_usage(const string& prog_name, ostream& out)
     << " --redundant  display redundant changes\n"
     << " --no-redundant  do not display redundant changes "
     "(this is the default)\n"
-    << " --impacted-interfaces  do not display interfaces impacted"
-    " by leaf changes\n"
+    << " --impacted-interfaces  display interfaces impacted by leaf changes\n"
     << " --dump-diff-tree  emit a debug dump of the internal diff tree to "
     "the error output stream\n"
     <<  " --stats  show statistics about various internal stuff\n"
+#ifdef WITH_DEBUG_SELF_COMPARISON
+    << " --debug debug the process of comparing an ABI corpus against itself"
+#endif
     << " --verbose show verbose messages about internal stuff\n";
 }
 
@@ -315,7 +315,22 @@ parse_command_line(int argc, char* argv[], options& opts)
 	      opts.wrong_option = argv[i];
 	      return true;
 	    }
-	  opts.headers_dir1 = argv[j];
+	  // The user can specify several header files directories for
+	  // the first binary.
+	  opts.headers_dirs1.push_back(argv[j]);
+	  ++i;
+	}
+      else if (!strcmp(argv[i], "--header-file1")
+	       || !strcmp(argv[i], "--hf1"))
+	{
+	  int j = i + 1;
+	  if (j >= argc)
+	    {
+	      opts.missing_operand = true;
+	      opts.wrong_option = argv[i];
+	      return true;
+	    }
+	  opts.header_files1.push_back(argv[j]);
 	  ++i;
 	}
       else if (!strcmp(argv[i], "--headers-dir2")
@@ -328,7 +343,22 @@ parse_command_line(int argc, char* argv[], options& opts)
 	      opts.wrong_option = argv[i];
 	      return true;
 	    }
-	  opts.headers_dir2 = argv[j];
+	  // The user can specify several header files directories for
+	  // the first binary.
+	  opts.headers_dirs2.push_back(argv[j]);
+	  ++i;
+	}
+      else if (!strcmp(argv[i], "--header-file2")
+	       || !strcmp(argv[i], "--hf2"))
+	{
+	  int j = i + 1;
+	  if (j >= argc)
+	    {
+	      opts.missing_operand = true;
+	      opts.wrong_option = argv[i];
+	      return true;
+	    }
+	  opts.header_files2.push_back(argv[j]);
 	  ++i;
 	}
       else if (!strcmp(argv[i], "--kmi-whitelist")
@@ -549,6 +579,10 @@ parse_command_line(int argc, char* argv[], options& opts)
 	opts.show_stats = true;
       else if (!strcmp(argv[i], "--verbose"))
 	opts.do_log = true;
+#ifdef WITH_DEBUG_SELF_COMPARISON
+      else if (!strcmp(argv[i], "--debug"))
+	opts.do_debug = true;
+#endif
       else
 	{
 	  if (strlen(argv[i]) >= 2 && argv[i][0] == '-' && argv[i][1] == '-')
@@ -649,7 +683,19 @@ set_diff_context_from_opts(diff_context_sptr ctxt,
   ctxt->show_added_vars(opts.show_all_vars || opts.show_added_vars);
   ctxt->show_linkage_names(opts.show_linkage_names);
   ctxt->show_locs(opts.show_locs);
-  ctxt->show_redundant_changes(opts.show_redundant_changes);
+  // So when we are showing only leaf changes, we want to show
+  // redundant changes because of this: Suppose several functions have
+  // their return type changed from void* to int*.  We want them all
+  // to be reported.  In that case the change is not redundant.  As
+  // far as user-defined type changes (like struct/class) they are
+  // already put inside a map which makes them be non-redundant, so we
+  // don't have to worry about that case.
+  //
+  // TODO: maybe that in this case we should avoid firing the
+  // redundancy analysis pass altogether.  That could help save a
+  // couple of CPU cycle here and there!
+  ctxt->show_redundant_changes(opts.show_redundant_changes
+                               || opts.leaf_changes_only);
   ctxt->show_symbols_unreferenced_by_debug_info
     (opts.show_symbols_not_referenced_by_debug_info);
   ctxt->show_added_symbols_unreferenced_by_debug_info
@@ -679,22 +725,22 @@ set_diff_context_from_opts(diff_context_sptr ctxt,
       load_default_user_suppressions(supprs);
     }
 
-  if (!opts.headers_dir1.empty())
+  if (!opts.headers_dirs1.empty() || !opts.header_files1.empty())
     {
       // Generate suppression specification to avoid showing ABI
       // changes on types that are not defined in public headers.
       suppression_sptr suppr =
-	gen_suppr_spec_from_headers(opts.headers_dir1);
+	gen_suppr_spec_from_headers(opts.headers_dirs1, opts.header_files1);
       if (suppr)
 	ctxt->add_suppression(suppr);
     }
 
-    if (!opts.headers_dir2.empty())
+  if (!opts.headers_dirs2.empty() || !opts.header_files2.empty())
     {
       // Generate suppression specification to avoid showing ABI
       // changes on types that are not defined in public headers.
       suppression_sptr suppr =
-	gen_suppr_spec_from_headers(opts.headers_dir2);
+	gen_suppr_spec_from_headers(opts.headers_dirs2, opts.header_files2);
       if (suppr)
 	ctxt->add_suppression(suppr);
     }
@@ -729,7 +775,7 @@ set_suppressions(ReadContextType& read_ctxt, const options& opts)
     read_suppressions(*i, supprs);
 
   if (read_context_get_path(read_ctxt) == opts.file1
-      && !opts.headers_dir1.empty())
+      && (!opts.headers_dirs1.empty() || !opts.header_files1.empty()))
     {
       // Generate suppression specification to avoid showing ABI
       // changes on types that are not defined in public headers for
@@ -739,7 +785,7 @@ set_suppressions(ReadContextType& read_ctxt, const options& opts)
       // corpus loading, they are going to be dropped from the
       // internal representation altogether.
       suppression_sptr suppr =
-	gen_suppr_spec_from_headers(opts.headers_dir1);
+	gen_suppr_spec_from_headers(opts.headers_dirs1, opts.header_files1);
       if (suppr)
 	{
 	  if (opts.drop_private_types)
@@ -749,7 +795,7 @@ set_suppressions(ReadContextType& read_ctxt, const options& opts)
     }
 
   if (read_context_get_path(read_ctxt) == opts.file2
-      && !opts.headers_dir2.empty())
+      && (!opts.headers_dirs2.empty() || !opts.header_files2.empty()))
     {
       // Generate suppression specification to avoid showing ABI
       // changes on types that are not defined in public headers for
@@ -759,7 +805,7 @@ set_suppressions(ReadContextType& read_ctxt, const options& opts)
       // corpus loading, they are going to be dropped from the
       // internal representation altogether.
       suppression_sptr suppr =
-	gen_suppr_spec_from_headers(opts.headers_dir2);
+	gen_suppr_spec_from_headers(opts.headers_dirs2, opts.header_files2);
       if (suppr)
 	{
 	  if (opts.drop_private_types)
@@ -974,6 +1020,26 @@ handle_error(abigail::dwarf_reader::status status_code,
   return abigail::tools_utils::ABIDIFF_OK;
 }
 
+/// Emit an error message saying that the two files have incompatible
+/// format versions.
+///
+/// @param file_path1 the first file path to consider.
+///
+/// @param file_path2 the second file path to consider.
+///
+/// @param prog_name the name of the current program.
+static void
+emit_incompatible_format_version_error_message(const string& file_path1,
+					       const string& file_path2,
+					       const string& prog_name)
+{
+  emit_prefix(prog_name, cerr)
+    << "incompatible format version between the two input files:\n"
+    << "'" << file_path1 << "'\n"
+    << "and\n"
+    << "'" << file_path2 << "'\n" ;
+}
+
 int
 main(int argc, char* argv[])
 {
@@ -1046,6 +1112,10 @@ main(int argc, char* argv[])
 	}
 
       environment_sptr env(new environment);
+#ifdef WITH_DEBUG_SELF_COMPARISON
+	    if (opts.do_debug)
+	      env->self_comparison_debug_is_on(true);
+#endif
       translation_unit_sptr t1, t2;
       abigail::dwarf_reader::status c1_status =
 	abigail::dwarf_reader::STATUS_OK,
@@ -1127,13 +1197,6 @@ main(int argc, char* argv[])
 				  argv[0], opts);
 	  }
 	  break;
-	case abigail::tools_utils::FILE_TYPE_ZIP_CORPUS:
-#ifdef WITH_ZIP_ARCHIVE
-	  c1 = abigail::xml_reader::read_corpus_from_file(opts.file1);
-	  if (!c1)
-	    return handle_error(c1_status, /*ctxt=*/0, argv[0], opts);
-#endif //WITH_ZIP_ARCHIVE
-	  break;
 	case abigail::tools_utils::FILE_TYPE_RPM:
 	case abigail::tools_utils::FILE_TYPE_SRPM:
 	case abigail::tools_utils::FILE_TYPE_DEB:
@@ -1202,13 +1265,6 @@ main(int argc, char* argv[])
 	      return handle_error(c2_status, /*ctxt=*/0, argv[0], opts);
 	  }
 	  break;
-	case abigail::tools_utils::FILE_TYPE_ZIP_CORPUS:
-#ifdef WITH_ZIP_ARCHIVE
-	  c2 = abigail::xml_reader::read_corpus_from_file(opts.file2);
-	  if (!c2)
-	    return handle_error(c2_status, /*ctxt=*/0, argv[0], opts);
-#endif //WITH_ZIP_ARCHIVE
-	  break;
 	case abigail::tools_utils::FILE_TYPE_RPM:
 	case abigail::tools_utils::FILE_TYPE_SRPM:
 	case abigail::tools_utils::FILE_TYPE_DEB:
@@ -1255,6 +1311,15 @@ main(int argc, char* argv[])
 	      return abigail::tools_utils::ABIDIFF_OK;
 	    }
 
+	  if (c1->get_format_major_version_number()
+	      != c2->get_format_major_version_number())
+	    {
+	      emit_incompatible_format_version_error_message(opts.file1,
+							     opts.file2,
+							     argv[0]);
+	      return abigail::tools_utils::ABIDIFF_ERROR;
+	    }
+
 	  set_corpus_keep_drop_regex_patterns(opts, c1);
 	  set_corpus_keep_drop_regex_patterns(opts, c2);
 
@@ -1277,6 +1342,15 @@ main(int argc, char* argv[])
 	      return abigail::tools_utils::ABIDIFF_OK;
 	    }
 
+	  if (g1->get_format_major_version_number()
+	      != g2->get_format_major_version_number())
+	    {
+	      emit_incompatible_format_version_error_message(opts.file1,
+							     opts.file2,
+							     argv[0]);
+	      return abigail::tools_utils::ABIDIFF_ERROR;
+	    }
+
 	  adjust_diff_context_for_kmidiff(*ctxt);
 	  corpus_diff_sptr diff = compute_diff(g1, g2, ctxt);
 
