@@ -1,23 +1,8 @@
+// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 // -*- Mode: C++ -*-
 //
 // Copyright (C) 2013-2020 Red Hat, Inc.
 //
-// This file is part of the GNU Application Binary Interface Generic
-// Analysis and Instrumentation Library (libabigail).  This library is
-// free software; you can redistribute it and/or modify it under the
-// terms of the GNU Lesser General Public License as published by the
-// Free Software Foundation; either version 3, or (at your option) any
-// later version.
-
-// This library is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// General Lesser Public License for more details.
-
-// You should have received a copy of the GNU Lesser General Public
-// License along with this program; see the file COPYING-LGPLV3.  If
-// not, see <http://www.gnu.org/licenses/>.
-//
 // Author: Dodji Seketeli
 
 /// @file
@@ -26,16 +11,17 @@
 /// DWARF format) and emit it back in a set of "text sections" in native
 /// libabigail XML format.
 
+#include "config.h"
 #include <unistd.h>
 #include <cassert>
-#include <cstring>
 #include <cstdio>
 #include <cstdlib>
-#include <string>
-#include <iostream>
+#include <cstring>
 #include <fstream>
+#include <iostream>
+#include <memory>
+#include <string>
 #include <vector>
-#include "abg-cxx-compat.h"
 #include "abg-config.h"
 #include "abg-tools-utils.h"
 #include "abg-corpus.h"
@@ -50,7 +36,7 @@ using std::cout;
 using std::ostream;
 using std::ofstream;
 using std::vector;
-using abg_compat::shared_ptr;
+using std::shared_ptr;
 using abigail::tools_utils::emit_prefix;
 using abigail::tools_utils::temp_file;
 using abigail::tools_utils::temp_file_sptr;
@@ -77,6 +63,7 @@ using abigail::xml_writer::type_id_style_kind;
 using abigail::xml_writer::write_context_sptr;
 using abigail::xml_writer::write_corpus;
 using abigail::xml_reader::read_corpus_from_native_xml_file;
+using abigail::xml_reader::create_native_xml_read_context;
 using abigail::dwarf_reader::read_context;
 using abigail::dwarf_reader::read_context_sptr;
 using abigail::dwarf_reader::read_corpus_from_elf;
@@ -113,11 +100,17 @@ struct options
   bool			noout;
   bool			show_locs;
   bool			abidiff;
+#ifdef WITH_DEBUG_SELF_COMPARISON
+  bool			debug_abidiff;
+#endif
   bool			annotate;
   bool			do_log;
   bool			drop_private_types;
   bool			drop_undefined_syms;
   type_id_style_kind	type_id_style;
+#ifdef WITH_DEBUG_SELF_COMPARISON
+  string		type_id_file_path;
+#endif
 
   options()
     : display_version(),
@@ -137,6 +130,9 @@ struct options
       noout(),
       show_locs(true),
       abidiff(),
+#ifdef WITH_DEBUG_SELF_COMPARISON
+      debug_abidiff(),
+#endif
       annotate(),
       do_log(),
       drop_private_types(false),
@@ -197,6 +193,9 @@ display_usage(const string& prog_name, ostream& out)
     << "  --vmlinux <path>  the path to the vmlinux binary to consider to emit "
        "the ABI of the union of vmlinux and its modules\n"
     << "  --abidiff  compare the loaded ABI against itself\n"
+#ifdef WITH_DEBUG_SELF_COMPARISON
+    << "  --debug-abidiff  debug the process of comparing the loaded ABI against itself\n"
+#endif
     << "  --annotate  annotate the ABI artifacts emitted in the output\n"
     << "  --stats  show statistics about various internal stuff\n"
     << "  --verbose show verbose messages about internal stuff\n";
@@ -343,6 +342,13 @@ parse_command_line(int argc, char* argv[], options& opts)
 	opts.linux_kernel_mode = false;
       else if (!strcmp(argv[i], "--abidiff"))
 	opts.abidiff = true;
+#ifdef WITH_DEBUG_SELF_COMPARISON
+      else if (!strcmp(argv[i], "--debug-abidiff"))
+	{
+	  opts.abidiff = true;
+	  opts.debug_abidiff = true;
+	}
+#endif
       else if (!strcmp(argv[i], "--annotate"))
 	opts.annotate = true;
       else if (!strcmp(argv[i], "--stats"))
@@ -482,11 +488,16 @@ static int
 load_corpus_and_write_abixml(char* argv[],
 			     environment_sptr& env,
 			     read_context_sptr& context,
-			     const options& opts)
+			     options& opts)
 {
   int exit_code = 0;
   timer t;
 
+#ifdef WITH_DEBUG_SELF_COMPARISON
+  if (opts.debug_abidiff)
+    env->self_comparison_debug_is_on(true);
+#endif
+
   read_context& ctxt = *context;
   corpus_sptr corp;
   dwarf_reader::status s = dwarf_reader::STATUS_UNKNOWN;
@@ -566,10 +577,28 @@ load_corpus_and_write_abixml(char* argv[],
 	  set_ostream(*write_ctxt, tmp_file->get_stream());
 	  write_corpus(*write_ctxt, corp, 0);
 	  tmp_file->get_stream().flush();
+
+#ifdef WITH_DEBUG_SELF_COMPARISON
+	  if (opts.debug_abidiff)
+	    {
+	      opts.type_id_file_path = tmp_file->get_path() + string(".typeid");
+	      write_canonical_type_ids(*write_ctxt, opts.type_id_file_path);
+	    }
+#endif
+	  xml_reader::read_context_sptr read_ctxt =
+	    create_native_xml_read_context(tmp_file->get_path(), env.get());
+
+#ifdef WITH_DEBUG_SELF_COMPARISON
+	  if (opts.debug_abidiff
+	      && !opts.type_id_file_path.empty())
+	    {
+	      load_canonical_type_ids(*read_ctxt, opts.type_id_file_path);
+	      remove(opts.type_id_file_path.c_str());
+	    }
+#endif
 	  t.start();
 	  corpus_sptr corp2 =
-	    read_corpus_from_native_xml_file(tmp_file->get_path(),
-					     env.get());
+	    read_corpus_from_input(*read_ctxt);
 	  t.stop();
 	  if (opts.do_log)
 	    emit_prefix(argv[0], cerr)
@@ -829,8 +858,6 @@ main(int argc, char* argv[])
       set_show_stats(ctxt, opts.show_stats);
       set_suppressions(ctxt, opts);
       abigail::dwarf_reader::set_do_log(ctxt, opts.do_log);
-      if (!opts.kabi_whitelist_supprs.empty())
-	set_ignore_symbol_table(ctxt, true);
 
       if (opts.check_alt_debug_info_path)
 	{
