@@ -1,22 +1,7 @@
+// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2020 Red Hat, Inc.
-//
-// This file is part of the GNU Application Binary Interface Generic
-// Analysis and Instrumentation Library (libabigail).  This library is
-// free software; you can redistribute it and/or modify it under the
-// terms of the GNU Lesser General Public License as published by the
-// Free Software Foundation; either version 3, or (at your option) any
-// later version.
-
-// This library is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// General Lesser Public License for more details.
-
-// You should have received a copy of the GNU Lesser General Public
-// License along with this program; see the file COPYING-LGPLV3.  If
-// not, see <http://www.gnu.org/licenses/>.
+// Copyright (C) 2013-2021 Red Hat, Inc.
 //
 // Author: Dodji Seketeli
 
@@ -28,14 +13,14 @@
 /// runs a diff on the two files and expects the result of the diff to
 /// be empty.
 
-#include <cstring>
 #include <cstdio>
 #include <cstdlib>
-#include <string>
-#include <iostream>
+#include <cstring>
 #include <fstream>
+#include <iostream>
+#include <memory>
+#include <string>
 #include <vector>
-#include "abg-cxx-compat.h"
 #include "abg-config.h"
 #include "abg-tools-utils.h"
 #include "abg-ir.h"
@@ -63,7 +48,6 @@ using abigail::corpus;
 using abigail::corpus_sptr;
 using abigail::xml_reader::read_translation_unit_from_file;
 using abigail::xml_reader::read_translation_unit_from_istream;
-using abigail::xml_reader::read_corpus_from_file;
 using abigail::xml_reader::read_corpus_from_native_xml;
 using abigail::xml_reader::read_corpus_from_native_xml_file;
 using abigail::xml_reader::read_corpus_group_from_input;
@@ -83,7 +67,7 @@ struct options
   bool				read_tu;
   bool				diff;
   bool				noout;
-  abg_compat::shared_ptr<char>	di_root_path;
+  std::shared_ptr<char>	di_root_path;
   vector<string>		suppression_paths;
   string			headers_dir;
   vector<string>		header_files;
@@ -383,11 +367,6 @@ main(int argc, char* argv[])
 	    group = read_corpus_group_from_input(*ctxt);
 	  }
 	  break;
-	case abigail::tools_utils::FILE_TYPE_ZIP_CORPUS:
-#if WITH_ZIP_ARCHIVE
-	  corp = read_corpus_from_file(opts.file_path);
-#endif
-	  break;
 	case abigail::tools_utils::FILE_TYPE_RPM:
 	  break;
 	case abigail::tools_utils::FILE_TYPE_SRPM:
@@ -461,18 +440,16 @@ main(int argc, char* argv[])
       else
 	{
 	  if (type == abigail::tools_utils::FILE_TYPE_XML_CORPUS
-	      ||type == abigail::tools_utils::FILE_TYPE_XML_CORPUS_GROUP
+	      || type == abigail::tools_utils::FILE_TYPE_XML_CORPUS_GROUP
 	      || type == abigail::tools_utils::FILE_TYPE_ELF)
 	    {
 	      if (!opts.noout)
-		is_ok = write_corpus(*ctxt, corp, 0);
-	    }
-	  else if (type == abigail::tools_utils::FILE_TYPE_ZIP_CORPUS)
-	    {
-#ifdef WITH_ZIP_ARCHIVE
-	      if (!opts.noout)
-		is_ok = write_corpus_to_archive(*corp, tmp_file->get_path());
-#endif //WITH_ZIP_ARCHIVE
+		{
+		  if (corp)
+		    is_ok = write_corpus(*ctxt, corp, 0);
+		  else if (group)
+		    is_ok = write_corpus_group(*ctxt, group, 0);
+		}
 	    }
 	}
 
@@ -491,8 +468,7 @@ main(int argc, char* argv[])
 	  && opts.diff
 	  && ((type == abigail::tools_utils::FILE_TYPE_XML_CORPUS)
 	      ||type == abigail::tools_utils::FILE_TYPE_XML_CORPUS_GROUP
-	      || type == abigail::tools_utils::FILE_TYPE_NATIVE_BI
-	      || type == abigail::tools_utils::FILE_TYPE_ZIP_CORPUS))
+	      || type == abigail::tools_utils::FILE_TYPE_NATIVE_BI))
 	{
 	  string cmd = "diff -u " + opts.file_path + " " + tmp_file->get_path();
 	  if (system(cmd.c_str()))
